#--------------------------------------------------------------------------
#     This file is part of BKchem - a chemical drawing program
#     Copyright (C) 2002  Beda Kosata <kosatab@vscht.cz>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------
#
# Last edited: $Date: 2002/10/20 20:11:05 $
#
#--------------------------------------------------------------------------

"""set of basic vector graphics classes such as rect, oval etc."""

import helper_graphics as hg
import dom_extensions
import xml.dom.minidom as dom
import misc
import classes
import operator

class vector_graphics_item:

  def __init__( self, paper, coords=(), package=None, outline='black', fill='white', width=1):
    self.object_type = 'vector'
    self.paper = paper
    self.selector = None
    self.outline = outline
    self.fill = fill
    self.item = None
    self.width = width
    self.coords = []
    if coords:
      self.coords = coords
    if package:
      self.read_package( package)

  def draw( self):
    pass

  def delete( self):
    self.paper.delete( self.item)
    self.paper.unregister_id( self.item) 
    self.item = None
    if self.selector:
      self.selector.delete()
      self.selector = None
    
  def focus( self):
    pass

  def unfocus( self):
    pass

  def select( self):
    self.selector = hg.selection_rect( self.paper, self, coords=tuple( self.coords), resize_event=self.resize, move_event=self.move)

  def unselect( self):
    if self.selector:
      self.selector.delete()
      self.selector = None
  
  def resize( self, coords, fix=()):
    self.coords = misc.normalize_coords( coords)
    self.paper.coords( self.item, self.coords)

  def move( self, dx, dy):
    self.paper.move( self.item, dx, dy)
    c = self.coords
    self.coords = ( c[0]+dx, c[1]+dy, c[2]+dx, c[3]+dy)
    if self.selector:
      self.selector.move( dx, dy)

  def get_package( self):
    pass
                           
  def read_package( self, pack):
    pass

  def bbox( self):
    return self.coords

  def lift( self):
    if self.item:
      self.paper.lift( self.item)
    if self.selector:
      self.selector.lift()


# RECTANGLE
class rect( vector_graphics_item):

  def __init__( self, paper, coords=(), package=None, outline='black', fill='white', width=1):
    vector_graphics_item.__init__( self, paper, coords=coords, package=package, outline=outline, fill=fill, width=width)
    self.object_type = 'rect'

  def draw( self):
    self.item = self.paper.create_rectangle( tuple( self.coords),
                                             fill = self.fill,
                                             outline = self.outline,
                                             width = self.width,
                                             tags=("rect","vector"))
    self.paper.register_id( self.item, self)

  def redraw( self):
    if not self.item:
      self.draw()
    else:
      self.paper.coords( self.item, tuple( self.coords))
      self.paper.itemconfig( self.item, width=self.width, fill=self.fill, outline=self.outline)
    
  def get_package( self):
    pack = dom.Element( 'rect')
    x1, y1, x2, y2 = self.paper.screen_to_real_coords( self.coords)
    dom_extensions.setAttributes( pack, (('x1', str( x1)),
                                         ('y1', str( y1)),
                                         ('x2', str( x2)),
                                         ('y2', str( y2)),
                                         ('outline', self.outline),
                                         ('fill', self.fill),
                                         ('width', str( self.width))))
    return pack
                           
  def read_package( self, pack):
    self.coords = self.paper.real_to_screen_coords( map( float, dom_extensions.getAttributes( pack, ['x1', 'y1', 'x2', 'y2'])))
    for n in ('outline', 'fill'):
      self.__dict__[ n] = pack.getAttribute( n)
    w = pack.getAttribute( 'width')
    if w:
      self.width = float( w)
    else:
      self.width = 1.0

# SQUARE
class square( rect):

  def get_package( self):
    pack = dom.Element( 'square')
    x1, y1, x2, y2 = self.paper.screen_to_real_coords( self.coords)
    dom_extensions.setAttributes( pack, (('x1', str( x1)),
                                         ('y1', str( y1)),
                                         ('x2', str( x2)),
                                         ('y2', str( y2)),
                                         ('outline', self.outline),
                                         ('fill', self.fill),
                                         ('width', str( self.width))))
    return pack
                           
  def select( self):
    self.selector = hg.selection_square( self.paper, self, coords=tuple( self.coords))

  def resize( self, coords, fix=()):
    if not fix:
      x1, y1, x2, y2 = misc.normalize_coords( coords)
      dx = x2 - x1
      dy = y2 - y1
      d = (abs( dx) + abs( dy))/2
      self.coords = (x1, y1, x1+d, y1+d)
    else:
      x1, y1, x2, y2 = coords
      dx = (fix[0] - x1) or (fix[0] - x2)
      dy = (fix[1] - y2) or (fix[1] - y1)
      d = (abs( dx) + abs( dy))/2
      self.coords = misc.normalize_coords( (fix[0], fix[1], x1-(d*misc.signum( dx) or d), y1-( d*misc.signum( dy) or d)))
    self.paper.coords( self.item, self.coords)


# OVAL
class oval( vector_graphics_item):

  def __init__( self, paper, coords=(), package=None, outline='black', fill='white'):
    vector_graphics_item.__init__( self, paper, coords=coords, package=package, outline=outline, fill=fill)
    self.object_type = 'oval'

  def draw( self):
    self.item = self.paper.create_oval( tuple( self.coords),
                                        fill = self.fill,
                                        outline = self.outline,
                                        width = self.width,
                                        tags=("oval","vector"))
    self.paper.register_id( self.item, self)

  def redraw( self):
    if not self.item:
      self.draw()
    else:
      self.paper.coords( self.item, tuple( self.coords))
      self.paper.itemconfig( self.item, width=self.width, fill=self.fill, outline=self.outline)
      
  def get_package( self):
    pack = dom.Element( 'oval')
    x1, y1, x2, y2 = self.paper.screen_to_real_coords( self.coords)
    dom_extensions.setAttributes( pack, (('x1', str( x1)),
                                         ('y1', str( y1)),
                                         ('x2', str( x2)),
                                         ('y2', str( y2)),
                                         ('outline', self.outline),
                                         ('fill', self.fill),
                                         ('width', str( self.width))))
    return pack
                           
  def read_package( self, pack):
    self.coords = self.paper.real_to_screen_coords( map( float, dom_extensions.getAttributes( pack, ['x1', 'y1', 'x2', 'y2'])))
    for n in ('outline', 'fill'):
      self.__dict__[ n] = pack.getAttribute( n)
    w = pack.getAttribute( 'width')
    if w:
      self.width = float( w)
    else:
      self.width = 1.0

  
# CIRCLE
class circle( oval):

  def get_package( self):
    pack = dom.Element( 'circle')
    x1, y1, x2, y2 = self.paper.screen_to_real_coords( self.coords)
    dom_extensions.setAttributes( pack, (('x1', str( x1)),
                                         ('y1', str( y1)),
                                         ('x2', str( x2)),
                                         ('y2', str( y2)),
                                         ('outline', self.outline),
                                         ('fill', self.fill),
                                         ('width', str( self.width))))
    return pack
                           
  def select( self):
    self.selector = hg.selection_square( self.paper, self, coords=tuple( self.coords), resize_event=self.resize, move_event=self.move)

  def resize( self, coords, fix=()):
    if not fix:
      x1, y1, x2, y2 = misc.normalize_coords( coords)
      dx = x2 - x1
      dy = y2 - y1
      d = (abs( dx) + abs( dy))/2
      self.coords = (x1, y1, x1+d, y1+d)
    else:
      x1, y1, x2, y2 = coords
      dx = (fix[0] - x1) or (fix[0] - x2)
      dy = (fix[1] - y2) or (fix[1] - y1)
      d = (abs( dx) + abs( dy))/2
      self.coords = misc.normalize_coords( (fix[0], fix[1], x1-(d*misc.signum( dx) or d), y1-( d*misc.signum( dy) or d)))
    self.paper.coords( self.item, self.coords)


# POLYGON
class polygon( vector_graphics_item):

  def __init__( self, paper, coords=(), package=None, outline='black', fill='white', width=1):
    vector_graphics_item.__init__( self, paper, coords=coords, package=package, outline=outline, fill=fill, width=width)
    del self.coords # polygon does use points for storage of coord information
    self.object_type = 'polygon'
    if not package:
      self.points = []
      for i in range( 0, len( coords), 2):
        x = coords[i]
        y = coords[i+1]
        self.points.append( classes.point( self.paper, xy=( x, y), arrow=self))
    

  def draw( self):
    [p.draw() for p in self.points]
    coords = reduce( operator.add, map( lambda b: b.get_xy(), self.points))
    self.item = self.paper.create_polygon( tuple( coords),
                                           fill=self.fill,
                                           outline=self.outline,
                                           width=self.width,
                                           tags=("polygon","vector"))
    self.paper.register_id( self.item, self)

  def move( self, dx, dy):
    [p.move( dx, dy) for p in self.points]
    self.paper.move( self.item, dx, dy)

  def redraw( self):
    if not self.item:
      self.draw()
    else:
      coords = reduce( operator.add, map( lambda b: b.get_xy(), self.points))
      self.paper.coords( self.item, tuple( coords))
      self.paper.itemconfig( self.item, width=self.width, fill=self.fill, outline=self.outline)

  def select( self):
    #self.selector = hg.selection_square( self.paper, self, coords=tuple( self.bbox()))
    [o.select() for o in self.points]

  def unselect( self):
    [o.unselect() for o in self.points]

  def bbox( self):
    return self.paper.bbox( self.item)

  def get_package( self):
    pack = dom.Element( 'polygon')
    dom_extensions.setAttributes( pack, (('outline', self.outline),
                                         ('fill', self.fill),
                                         ('width', str( self.width))))
    for p in self.points:
      pack.appendChild( p.get_package())
    return pack

  def read_package( self, pack):
    self.points = []
    for p in pack.getElementsByTagName( 'point'):
      self.points.append( classes.point( self.paper, arrow=self, package=p))
    for n in ('outline', 'fill'):
      self.__dict__[ n] = pack.getAttribute( n)
    w = pack.getAttribute( 'width')
    if w:
      self.width = float( w)
    else:
      self.width = 1.0
      
  def lift( self):
    if self.selector:
      self.selector.lift()
    if self.item:
      self.paper.lift( self.item)
    [o.lift() for o in self.points]

  def delete_point( self, pnt):
    try:
      self.points.remove( pnt)
    except IndexError:
      warn( "trying to remove nonexisting point from polygon")
    pnt.delete()

  def is_empty_or_single_point( self):
    return len( self.points) < 3 

  def delete( self):
    [o.delete() for o in self.points]
    self.points = []
    self.paper.delete( self.item)
    self.paper.unregister_id( self.item)
    self.item = None
