#--------------------------------------------------------------------------
#     This file is part of BKchem - a chemical drawing program
#     Copyright (C) 2002  Beda Kosata <kosatab@vscht.cz>

#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.

#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.

#     Complete text of GNU GPL can be found in the file gpl.txt in the
#     main directory of the program

#--------------------------------------------------------------------------
#
# Last edited: $Date: 2002/12/16 19:56:44 $
#
#--------------------------------------------------------------------------

"""set of marks such as charges, radicals etc."""

from __future__ import division
import geometry

class mark:

  def __init__( self, paper, x, y, size=4, fill='#ffffff', outline='#000', center=()):
    """size is a diameter of the mark, fill is the background color,
    outline is the color of lines and center is used for marks where
    not only position but also direction is needed"""
    self.paper = paper
    self.x = x
    self.y = y
    self.size = size
    self.items = []
    self.fill = fill
    self.outline = outline
    self.center = center
    self.draw()

  def draw( self):
    pass

  def delete( self):
    [self.paper.delete( o) for o in self.items]

  def move( self, dx, dy):
    self.x += dx
    self.y += dy
    [self.paper.move( o, dx, dy) for o in self.items]

  def move_to( self, x, y):
    dx = self.x - x
    dy = self.y - y
    self.move( dx, dy)

  def lift( self):
    [self.paper.lift( i) for i in self.items]
    

class radical( mark):

  def draw( self):
    """outline color is used for both outline and fill for radical"""
    x, y = self.x, self.y
    s = round( self.size / 2)
    self.items = [self.paper.create_oval( x-s, y-s, x+s, y+s,
                                          fill=self.outline, outline=self.outline,
                                          tags='mark')]


class biradical( mark):

  def draw( self):
    s = round( self.size / 2)
    x1, y1, x2, y2 = self.x, self.y, self.center[0], self.center[1]
    x, y = geometry.find_parallel( x1, y1, x2, y2, s*1.5)[0:2]
    # one circle
    self.items = [self.paper.create_oval( x-s, y-s, x+s, y+s,
                                          fill=self.outline, outline=self.outline,
                                          tags='mark')]
    # and the second on the other side
    x, y = geometry.find_parallel( x1, y1, x2, y2, -s*1.5)[0:2]
    self.items.append( self.paper.create_oval( x-s, y-s, x+s, y+s,
                                               fill=self.outline, outline=self.outline,
                                               tags='mark'))
    



class electronpair( mark):

  def __init__( self, paper, x, y, size=10, fill='#ffffff', outline='#000', center=()):
    mark.__init__( self, paper, x, y, size=size, fill=fill, outline=outline, center=center)

  def draw( self):
    s = round( self.size / 2)
    x1, y1, x2, y2 = self.x, self.y, self.center[0], self.center[1]
    # one end
    x, y = geometry.find_parallel( x1, y1, x2, y2, s)[0:2]
    # and the other
    x0, y0 = geometry.find_parallel( x1, y1, x2, y2, -s)[0:2]
    self.items = [self.paper.create_line( x, y, x0, y0, fill=self.outline,
                                          width=round(s/2), tags='mark')]
    


class plus( mark):

  def __init__( self, paper, x, y, size=10, fill='#ffffff', outline='#000', center=()):
    mark.__init__( self, paper, x, y, size=size, fill=fill, outline=outline, center=center)

  def draw( self):
    x, y = self.x, self.y
    s = round( self.size / 2)
    self.items = [self.paper.create_oval( x-s, y-s, x+s, y+s, fill=self.fill,
                                          outline=self.outline, tags='mark')]
    self.items.append( self.paper.create_line( x-s+2, y, x+s-1, y,
                                               fill=self.outline, tags='mark'))
    self.items.append( self.paper.create_line( x, y-s+2, x, y+s-1,
                                               fill=self.outline, tags='mark'))



class minus( mark):

  def __init__( self, paper, x, y, size=10, fill='#ffffff', outline='#000', center=()):
    mark.__init__( self, paper, x, y, size=size, fill=fill, outline=outline, center=center)

  def draw( self):
    x, y = self.x, self.y
    s = round( self.size / 2)
    self.items = [self.paper.create_oval( x-s, y-s, x+s, y+s, fill=self.fill,
                                          outline=self.outline, tags='mark')]
    self.items.append( self.paper.create_line( x-s+2, y, x+s-1, y,
                                               fill=self.outline, tags='mark'))



